#include <opencv2/opencv.hpp>

#include <iostream>
#include <fstream>

#include "database.hpp"

using namespace std;
using namespace cv;
using namespace cv::ml;


#define main_just_accuracy main
//#define main_with_statistics main
//#define main_with_histogram main


#ifdef  main_with_histogram
    #define features_size 1152
#else
    #define features_size 400
#endif

int main_just_accuracy() {



    Mat matTrainFeatures= Mat::zeros(num_train_sample,features_size, CV_32F);
    Mat matTestFeatures= Mat::zeros(num_test_sample,features_size,CV_32F);
    
    Mat matTrainLabels= Mat::zeros(1,num_train_sample,CV_32F);
    Mat matTestLabels= Mat::zeros(1,num_test_sample,CV_32F);

    
    //le as imagens
    //converte em linhas
    //cada linha tem 400 características
    //depois le os labels
    extractFeatures(matTrainFeatures, matTestFeatures, matTrainLabels, matTestLabels);
     
    //armazena os resultados
    //0,0 diz que será alocada pela quantidade de testes que existir
    Mat matResults(0,0,CV_32F);

    //cria uma amostra de treinamento
    Ptr<TrainData> trainingData;
    
    //instancia KNN
    Ptr<KNearest> kclassifier=KNearest::create();

    //associa treinamento com amostra criada
    //informa base em linhas ROW_SAMPLE (poderia ser colunas)
    trainingData=TrainData::create(matTrainFeatures,ROW_SAMPLE,matTrainLabels);

    //ajusta KNN como classificador (ele pode ser clusterizador tb)
    kclassifier->setIsClassifier(true);
    kclassifier->setAlgorithmType(KNearest::BRUTE_FORCE);

    //faz o treinamneto
    kclassifier->train(trainingData);
    
    for (int k=1; k<10; k++){
        kclassifier->setDefaultK(k); // valor de K
        
        //faz o teste!
        kclassifier->findNearest(matTestFeatures,kclassifier->getDefaultK(),matResults);

        float correto = 0;  
        for (int i=0; i<num_test_sample; i++) {
            if (matResults.at<float>(0,i) == matTestLabels.at<float>(0,i))  //se for igual, match!
                correto +=1;
        }
        cout << "k = " <<k << " -> ";
        cout << "Acurácia: " << correto / (float)num_test_sample << endl;
    }
}


int main_with_statistics() {


    Mat matTrainFeatures= Mat::zeros(num_train_sample,features_size, CV_32F);
    Mat matTestFeatures= Mat::zeros(num_test_sample,features_size,CV_32F);
    
    Mat matTrainLabels= Mat::zeros(1,num_train_sample,CV_32F);
    Mat matTestLabels= Mat::zeros(1,num_test_sample,CV_32F);

    
    //le as imagens
    //converte em linhas
    //cada linha tem 400 características
    //depois le os labels
    extractFeatures(matTrainFeatures, matTestFeatures, matTrainLabels, matTestLabels);
     
    //armazena os resultados
    //0,0 diz que será alocada pela quantidade de testes que existir
    Mat matResults(0,0,CV_32F);

    //cria uma amostra de treinamento
    Ptr<TrainData> trainingData;
    
    //instancia KNN
    Ptr<KNearest> kclassifier=KNearest::create();

    //associa treinamento com amostra criada
    //informa base em linhas ROW_SAMPLE (poderia ser colunas)
    trainingData=TrainData::create(matTrainFeatures,ROW_SAMPLE,matTrainLabels);

    //ajusta KNN como classificador (ele pode ser clusterizador tb)
    kclassifier->setIsClassifier(true);
    kclassifier->setAlgorithmType(KNearest::BRUTE_FORCE);

    //faz o treinamneto
    kclassifier->train(trainingData);
    float correto[10];
    float qtd[10];
    
    for (int k=1; k<10; k++){
        kclassifier->setDefaultK(k); // valor de K
        
        //faz o teste!
        kclassifier->findNearest(matTestFeatures,kclassifier->getDefaultK(),matResults);

        
        for (int i=0; i<10; i++) correto[i] = qtd[i] = 0;
        
        for (int i=0; i<num_test_sample; i++) {
            if (matResults.at<float>(0,i) == matTestLabels.at<float>(0,i))  //se for igual, match!
                correto[(int)matResults.at<float>(0,i)] +=1;
            
            qtd[(int)matTestLabels.at<float>(0,i)] +=1;
            
        }
        cout << "k = " <<k << " -> " << endl;
        
        cout << "Acurácia por digito: " << endl;
        for (int i=0; i<10; i++)
            cout << "(" << i << "): " << correto[i]/qtd[i] << endl;
    }
}


int main_with_histogram() {

    Mat matTrainFeatures= Mat::zeros(num_train_sample,features_size, CV_32F);
    Mat matTestFeatures= Mat::zeros(num_test_sample,features_size,CV_32F);
    
    Mat matTrainLabels= Mat::zeros(1,num_train_sample,CV_32F);
    Mat matTestLabels= Mat::zeros(1,num_test_sample,CV_32F);

    
    //le as imagens
    //converte em linhas
    //cada linha tem 400 características
    //depois le os labels
    extractFeaturesHOG(matTrainFeatures, matTestFeatures, matTrainLabels, matTestLabels);
     
    //armazena os resultados
    //0,0 diz que será alocada pela quantidade de testes que existir
    Mat matResults(0,0,CV_32F);

    //cria uma amostra de treinamento
    Ptr<TrainData> trainingData;
    
    //instancia KNN
    Ptr<KNearest> kclassifier=KNearest::create();

    //associa treinamento com amostra criada
    //informa base em linhas ROW_SAMPLE (poderia ser colunas)
    trainingData=TrainData::create(matTrainFeatures,ROW_SAMPLE,matTrainLabels);

    //ajusta KNN como classificador (ele pode ser clusterizador tb)
    kclassifier->setIsClassifier(true);
    kclassifier->setAlgorithmType(KNearest::BRUTE_FORCE);

    //faz o treinamneto
    kclassifier->train(trainingData);
    float correto[10], sum=0;
    float qtd[10], all=0;
    
    for (int k=1; k<10; k++){
        kclassifier->setDefaultK(k); // valor de K
        
        //faz o teste!
        kclassifier->findNearest(matTestFeatures,kclassifier->getDefaultK(),matResults);

        
        for (int i=0; i<10; i++) correto[i] = qtd[i] = 0;
        
        for (int i=0; i<num_test_sample; i++) {
            if (matResults.at<float>(0,i) == matTestLabels.at<float>(0,i)) {
                correto[(int)matResults.at<float>(0,i)] +=1;
                sum +=1;
            }
            
            qtd[(int)matTestLabels.at<float>(0,i)] +=1;
            all +=1;
            
        }
        cout << "k = " <<k << " -> " << sum /all << endl;
        
        cout << "Acurácia por digito: " << endl;
        for (int i=0; i<10; i++)
            cout << "(" << i << "): " << correto[i]/qtd[i] << endl;
    }
}
